package gus05.manager.gus.endorf.layer3.loading;

import gus05.manager.gus.endorf.layer3.system1.System1;
import gus05.manager.gus.endorf.layer3.tools.Tools_collection;
import gus05.manager.gus.endorf.layer3.tools.Tools_file;
import gus05.manager.gus.endorf.layer3.tools.Tools_fileFilter;
import gus05.manager.gus.endorf.layer3.tools.Tools_jar;
import gus05.manager.gus.endorf.layer3.tools.Tools_zip;

import java.io.File;
import java.util.Iterator;
import java.util.Properties;


public class Loader_entities extends Loader {

    
    
    
    public Loader_entities()
    {super("entities");}
    
    
    
    
    
    
    
    protected void outsideLoad()
    {
        loadFromDir();
        loadFromFile();
    }
    
  
    
    
    
    
    protected void insideLoad()
    {
        try
        {
            Properties p = insideProp("load");
            if(p==null) return;
            
            initInfo();
            Iterator it = p.keySet().iterator();
            while(it.hasNext())
            {
                 String key = (String)it.next();
                 String value = p.getProperty(key);
                 loadEntity_fromLine(key,value);
            }
            debug("insideLoad()","load "+p.size()+" entities from inside");
        }
        catch(Exception e)
        {System1.printError(this,"insideLoad()",e);}
    }
    
    
    
    
    
    
    
    
    
    
    
    private void loadFromFile()
    {
        try
        {
            File f = outsideFile(System1.PATH_LOADFILE);
            if(f==null) return;
            
            Properties prop = Tools_file.prop(f);
            
            initInfo();
            Iterator it = prop.keySet().iterator();
            while(it.hasNext())
            {
                 String key = (String)it.next();
                 String value = prop.getProperty(key);
                 loadEntity_fromLine(key,value);
            }
            debug("loadFromFile()","load "+prop.size()+" entities from: "+f.getAbsolutePath());
        }
        catch(Exception e)
        {System1.printError(this,"loadFromFile()",e);}
    }
    
    
    
    
    
    
    
    private void loadFromDir()
    {
        try
        {
            File entityDir = outsideDir(System1.PATH_ENTITYDIR);
            if(entityDir==null)return;
            
            File[] zips = entityDir.listFiles(Tools_fileFilter.zipFileFilter);
            for(int i=0;i<zips.length;i++)
            initializeFromZip(zips[i],entityDir);
            
            File[] dirs = entityDir.listFiles(Tools_fileFilter.dirFileFilter);
            
            // load each entity
            for(int i=0;i<dirs.length;i++)
            loadEntity_fromDir(dirs[i]);
            
            debug("loadFromDir()","load "+dirs.length+" entities from: "+entityDir.getAbsolutePath());
        }
        catch(Exception e)
        {System1.printError(this,"loadFromDir()",e);}
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private void loadEntity_fromDir(File dir) throws Exception
    {
            String name = dir.getName();
            File jarFile = new File(dir,name+".jar");
            if(!jarFile.exists()) throw new Exception("jar file "+jarFile.getAbsolutePath()+" does not exist");
            
            // load properties file
            File propFile = new File(dir,"entity.properties");
            if(!propFile.exists()) throw new Exception("No prop file was found in directory "+dir.getName());
            Properties info = Tools_file.prop(propFile);
            
            // check external jars
            File jarsDir = new File(dir,"externaljars");
            if(jarsDir.exists() && jarsDir.isDirectory())
            {
                File[] jars = jarsDir.listFiles(Tools_fileFilter.jarFileFilter);
                for(int i=0;i<jars.length;i++) Tools_jar.checkExtJar(jars[i],name);
            }
            
            // check custom mapping
            File mappingFile = new File(dir,"mapping.properties");
            if(mappingFile.exists() && mappingFile.isFile())
            {
                Properties mappingProp = Tools_file.prop(mappingFile);
                Tools_collection.complete(System1.mapping, mappingProp);
            }
            
            // load entity class
            System1.entityManager.loadEntity_external(name,info,jarFile);
            increase();
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private void loadEntity_fromLine(String key, String value)
    {
        try
        {
            Properties info = new Properties();
            String classpath = getClasspath(value);
            String multiplicity = getMultiplicity(key,value);
            String author = currentAppUser;
            
            // set AUTHOR
            info.setProperty(System1.ENTITYINFO_AUTHOR,author); 
            // set SIGN
            info.setProperty(System1.ENTITYINFO_SIGN,"?"); 
            // set CLASSPATH
            info.setProperty(System1.ENTITYINFO_CLASSPATH,classpath); 
            // set JAVA VERSION
            info.setProperty(System1.ENTITYINFO_JAVAVERSION,currentJavaVersion); 
            // set MULTIPLICITY
            info.setProperty(System1.ENTITYINFO_MULTIPLICITY,multiplicity);
            // set CREATION DATE
            info.setProperty(System1.ENTITYINFO_CREATIONDATE,"?");
            // set DEPLOYING DATE
            info.setProperty(System1.ENTITYINFO_DEPLOYINGDATE,"?");
            // set DEPLOYER
            info.setProperty(System1.ENTITYINFO_DEPLOYER,"?");
            
            System1.entityManager.loadEntity(key,info,classpath);
            increase();
        }
        catch(Exception e)
        {
            System1.printMessage(this,"loadEntity_fromLine(String,String)","error for key="+key);
            System1.printError(this,"loadEntity_fromLine(String,String)",e);
        }
    }
    
    
    
    
    
    
    
    
    
    
    
    private void initializeFromZip(File zipFile, File outputDir)
    {
        try
        {
            String name = Tools_file.getFileName(zipFile);
            File dir = new File(outputDir,name);
            if(dir.isDirectory()) return;
            Tools_zip.unzip(zipFile,outputDir);
        }
        catch(Exception e)
        {System1.printError(this,"initializeFromZip(File,File)",e);}
    }
    
    
    
    
    
    private String currentAppUser;
    private String currentJavaVersion;
    
    private void initInfo()
    {
        currentAppUser = System1.getProperty(System1.PROPKEY_APPUSER,
            System1.ENTITYINFO_AUTHOR_UNKNOWN);
        currentJavaVersion = System.getProperty("java.version");
    }
    
    
    
    
    private String getMultiplicity(String key, String value) throws Exception
    {
        if(value.endsWith("*")) return System1.ENTITYINFO_MULTIPLICITYVALUE_MANY;
        if(value.endsWith("+")) return System1.ENTITYINFO_MULTIPLICITYVALUE_UNIQUE;
        throw new Exception("Entity "+key+" could not be loaded with such information: "+value);
    }
    
    private String getClasspath(String value)
    {
        if(value.endsWith("*") || value.endsWith("+"))
            value = value.substring(0,value.length()-1);
        if(!value.startsWith("gus05.entity."))
            value = "gus05.entity."+value;
        return value;
    }
}
