package gus05.manager.gus.endorf.layer3.system1;

import gus05.framework.features.Filter;
import gus05.manager.gus.endorf.layer3.tools.Tools_jar;
import gus05.manager.gus.endorf.layer3.tools.Tools_javaVersion;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.util.Properties;

import sun.misc.BASE64Decoder;


public class EntityManager {

    
    
    private BASE64Decoder decoder;   
    private Signature signature;
    
    
    public EntityManager()
    {init();}
    
    
    
    private void init()
    {
        try
        {
            signature = Signature.getInstance("DSA");
            decoder = new BASE64Decoder();
        }
        catch(Exception e)
        {System1.printError(this,"init()",e);}
    }
    
    
    
    
    
    
    
    
    private void load(String name, Properties info, Class c)
    {
        if(System1.isEntity(name))
            System1.printWarning(this,"load(String,Properties,Class)","entity "+name+" is loaded many times");
        System1.infoMap.put(name,info);
        System1.putEntityClass(name,c);
    }
    
    
    public void loadEntity(String name, Properties info, String classpath) throws Exception
    {
        if(!classpath.startsWith("gus05.entity."))
            classpath = "gus05.entity."+classpath;
        
        Class c = Class.forName(classpath);
        load(name,info,c);
    }
    
    
    public void loadEntity(String name, Properties info, String classpath, File jarFile) throws Exception
    {
        if(!classpath.startsWith("gus05.entity."))
            classpath = "gus05.entity."+classpath;
        
        Class c = Tools_jar.loadClass(jarFile,classpath);
        load(name,info,c);
    }
    
    
    
    
    
    


    
    
    

    public void loadEntity_external(String name, Properties info, File jarFile) throws Exception
    {
        String sign = info.getProperty(System1.ENTITYINFO_SIGN);
        String author = info.getProperty(System1.ENTITYINFO_AUTHOR);
        String classpath = info.getProperty(System1.ENTITYINFO_CLASSPATH);
        String javaVersion = info.getProperty(System1.ENTITYINFO_JAVAVERSION);
        
        if(sign==null) throw new Exception("sign not found in info for entity: "+name);
        if(author==null) throw new Exception("author not found in info for entity: "+name);
        if(classpath==null) throw new Exception("classpath is not found in info for entity: "+name);
        if(javaVersion==null) throw new Exception("java version not found in info for entity: "+name);
        
        if(!Tools_javaVersion.isJavaVersionSupported(javaVersion))
            throw new Exception("entity "+name+" has unsupported java version : "+javaVersion);
        
        if(!isEntitySecured(name,author,sign,jarFile))
            throw new Exception("Attempt to load insecure entity "+name);

        loadEntity(name,info,classpath,jarFile);
    }


    
    

    private boolean isEntitySecured(String entityName, String author, String sign, File jarFile) throws Exception
    {
        if(belongToSecuredList(entityName)) return true;
        return checkEntitySign(author,sign,jarFile);
    }

    
    
    
    

    private boolean belongToSecuredList(String entityName) throws Exception
    {
        Filter f = (Filter) System1.getInternal(System1.INTERNAL_OBJ_ENTITYSECUREDLIST);
        if(f==null) return true;
        return f.filter(entityName);
    }
    
    
    
    private boolean checkEntitySign(String author, String sign, File jarFile) throws Exception
    {
        if(!System1.pKeyMap.containsKey(author)) return false;
        PublicKey key = (PublicKey)System1.pKeyMap.get(author);
        signature.initVerify(key);
        updateSignature(jarFile);
        
        byte[] signData = decoder.decodeBuffer(sign);
        return signature.verify(signData);
    }
    
    
    
    
    
    private void updateSignature(File file) throws SignatureException, IOException
    {
        InputStream in = new FileInputStream(file);
        byte[] buffer = new byte[8192];
        int length;
        while((length=in.read(buffer))!=-1)
        signature.update(buffer, 0, length);
    }
}
