package gus05.manager.gus.endorf.layer3.tools;

import gus05.manager.gus.endorf.layer3.system1.System1;

import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.DSAPrivateKeySpec;
import java.security.spec.DSAPublicKeySpec;
import java.util.Properties;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;


public class Tools_publicKey {

    
    public static final String KEY_G = "G";
    public static final String KEY_P = "P";
    public static final String KEY_Q = "Q";
    public static final String KEY_Y = "Y";
    public static final String KEY_X = "X";
    public static final String KEY_OWNER = "owner";
    public static final String KEY_SIGN = "sign";
    
    
    private static KeyFactory kf;
    private static Signature si;
    private static BASE64Decoder de;   
    private static BASE64Encoder en;
    
    public static final String ALGORITHM = "DSA";
    
    
    private static void init() throws Exception
    {
        if(kf==null) kf = KeyFactory.getInstance(ALGORITHM);
        if(si==null) si = Signature.getInstance(ALGORITHM);
        if(de==null) de = new BASE64Decoder();
        if(en==null) en = new BASE64Encoder();
    }
    
    
    
    
    
    
    
    
    
    public final static PublicKey buildPublicKey(Properties prop) throws Exception
    {
        init();
        
        if(!prop.containsKey(KEY_G)) throw new Exception("Invalid public key properties: G is missing");
        if(!prop.containsKey(KEY_P)) throw new Exception("Invalid public key properties: P is missing");
        if(!prop.containsKey(KEY_Q)) throw new Exception("Invalid public key properties: Q is missing");
        if(!prop.containsKey(KEY_Y)) throw new Exception("Invalid public key properties: Y is missing");
        if(!prop.containsKey(KEY_OWNER)) throw new Exception("Invalid public key properties: OWNER is missing");
        if(!prop.containsKey(KEY_SIGN)) throw new Exception("Invalid public key properties: SIGN is missing");
        
        String g = prop.getProperty(KEY_G);
        String p = prop.getProperty(KEY_P);
        String q = prop.getProperty(KEY_Q);
        String y = prop.getProperty(KEY_Y);
        String owner = prop.getProperty(KEY_OWNER);
        String sign = prop.getProperty(KEY_SIGN);
        
        
        if(owner.equals("augustin.delale.06081977"))
        {
            String g_ = "5421644057436475141609648488325705128047428394380474376834667300766108262613900542681289080713724597310673074119355136085795982097390670890367185141189796";
            String q_ = "857393771208094202104259627990318636601332086981";
            String p_ = "13232376895198612407547930718267435757728527029623408872245156039757713029036368719146452186041204237350521785240337048752071462798273003935646236777459223";
            String y_ = "4517575272117675090020619265258517436262221026902616530261677237959284587872138777828846836705449477570567249122240682050678251594040682645116912167989747";
            
            if(!g.equals(g_)) corruptedPublicKey(owner);
            if(!q.equals(q_)) corruptedPublicKey(owner);
            if(!p.equals(p_)) corruptedPublicKey(owner);
            if(!y.equals(y_)) corruptedPublicKey(owner);
        }
        
        
        BigInteger G = new BigInteger(g);
        BigInteger P = new BigInteger(p);
        BigInteger Q = new BigInteger(q);
        BigInteger Y = new BigInteger(y);
        
        DSAPublicKeySpec keySpec = new DSAPublicKeySpec(Y,P,Q,G);
        PublicKey publicKey = kf.generatePublic(keySpec);
        
        
        si.initVerify(publicKey);
        si.update(owner.getBytes());
        byte[] signData = de.decodeBuffer(sign);
        boolean valid = si.verify(signData);
        if(!valid) corruptedPublicKey(owner);
        
        return publicKey;
    }
    
    
    
    
    
    
    
    
    public final static PrivateKey buildPrivateKey(Properties prop) throws Exception
    {
        init();
        
        if(!prop.containsKey(KEY_G)) throw new Exception("Invalid private key properties: G is missing");
        if(!prop.containsKey(KEY_P)) throw new Exception("Invalid private key properties: P is missing");
        if(!prop.containsKey(KEY_Q)) throw new Exception("Invalid private key properties: Q is missing");
        if(!prop.containsKey(KEY_X)) throw new Exception("Invalid private key properties: X is missing");
        if(!prop.containsKey(KEY_OWNER)) throw new Exception("Invalid private key properties: OWNER is missing");
        
        String g = prop.getProperty(KEY_G);
        String p = prop.getProperty(KEY_P);
        String q = prop.getProperty(KEY_Q);
        String x = prop.getProperty(KEY_X);
        
        BigInteger G = new BigInteger(g);
        BigInteger P = new BigInteger(p);
        BigInteger Q = new BigInteger(q);
        BigInteger X = new BigInteger(x);
        
        DSAPrivateKeySpec keySpec = new DSAPrivateKeySpec(X,P,Q,G);
        return kf.generatePrivate(keySpec);
    }
    
    
    
    
    
    
    
    public static Properties publicProp(PublicKey publicKey, String owner, byte[] signData) throws Exception
    {
        init();
        
        DSAPublicKeySpec spec = (DSAPublicKeySpec) kf.getKeySpec(publicKey,DSAPublicKeySpec.class);
        String sign = en.encode(signData);
        
        Properties prop = new Properties();
        prop.setProperty(KEY_G,spec.getG().toString());
        prop.setProperty(KEY_P,spec.getP().toString());
        prop.setProperty(KEY_Q,spec.getQ().toString());
        prop.setProperty(KEY_Y,spec.getY().toString());
        prop.setProperty(KEY_OWNER,owner);
        prop.setProperty(KEY_SIGN,sign);
        return prop;
    }
    
    
    
    
    
    
    
    
    public static Properties privateProp(PrivateKey privateKey, String owner, byte[] signData) throws Exception
    {
        init();
        
        DSAPrivateKeySpec spec = (DSAPrivateKeySpec) kf.getKeySpec(privateKey,DSAPrivateKeySpec.class);
        String sign = en.encode(signData);
        
        Properties prop = new Properties();
        prop.setProperty(KEY_G,spec.getG().toString());
        prop.setProperty(KEY_P,spec.getP().toString());
        prop.setProperty(KEY_Q,spec.getQ().toString());
        prop.setProperty(KEY_X,spec.getX().toString());
        prop.setProperty(KEY_OWNER,owner);
        prop.setProperty(KEY_SIGN,sign);
        return prop;
    }
    
    
    
    
    
    private static void corruptedPublicKey(String owner)
    {System1.FATAL_ERROR(Tools_publicKey.class,"corruptedPublicKey(String)","corrupted public key for owner "+owner);}
    
    
}
