package gus05.manager.gus.endorf.layer4.system2;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;
import java.net.URL;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import gus05.framework.features.Execute;
import gus05.framework.features.Give;
import gus05.framework.features.Provide;
import gus05.framework.features.Support;
import gus05.framework.features.Transform;
import gus05.manager.gus.endorf.layer2.datasupport.*;
import gus05.manager.gus.endorf.layer3.tools.Tools_file;
import gus05.manager.gus.endorf.layer3.tools.Tools_url;



public class DataManager implements Transform, Provide, Execute, ActionListener  {

    
    
    public SupportMap mapData;
    public SupportMap mapSupport;
    public SupportMap mapFiles;
    public SupportMap mapAccess;
    
    private HashSet modifiedData;
    
    
    
    
    public DataManager()
    {
        mapData = new SupportMap("mapData");
        mapSupport = new SupportMap("mapSupport");
        mapFiles = new SupportMap("mapFiles");
        mapAccess = new SupportMap("mapAccess");
        modifiedData = new HashSet();
        
        System2.finalTasks.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e)
            {saveModifiedData();}});
    }
    
    
    
    public Object transform(Object obj) throws Exception
    {
        String dataID = (String)obj;
        if(mapData.containsKey(dataID))
            return mapData.get(dataID);
        
        Support supportData = buildData(dataID);
        mapSupport.put(dataID,supportData);
        
        Object data = customizeData(dataID, supportData);
        mapData.put(dataID,data);
        
        return data;
    }
    
    
    
    public Object provide() throws Exception
    {return mapData;}
    
    
    
    
    
    
    
    
    
    
    private Support buildData(String dataID) throws Exception
    {
        try
        {
            if(!dataID.contains("#"))throw new Exception("Wrong dataID: "+dataID);
            if(dataID.contains(" ")) throw new Exception("Wrong dataID: "+dataID);
            
            String[] n = dataID.split("#",2);
            String dataType = n[0];
            String dataLocation = n[1];
            
            if(dataLocation.startsWith("path."))
                return buildDataFromFile(dataType, dataLocation, dataID);
            if(dataLocation.startsWith("pool."))
                return buildDataFromPool(dataType, dataLocation, dataID);
            if(dataLocation.startsWith("url."))
                return buildDataFromURL(dataType, dataLocation, dataID);
            
             return buildDataFromScratch(dataType, dataID);
        }
        catch(Exception e)
        {
            System2.printMessage(this,"buildData(String)","dataID "+dataID+" failed");
            throw e;
        }
    }
    
    
    
    
    
    
    
    
    private Support buildDataFromFile(String dataType, String dataLocation, String dataID) throws Exception
    {
        File file = System2.file(dataLocation);
        if(file==null) throw new Exception("Unknown data location: "+dataLocation);
        
        initDataAccess(dataID);
        
        if(dataType.equals(System2.RESOURCEPROVIDER_DATALIST))      return loadList_fromFile(dataID,file);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATASET))       return loadSet_fromFile(dataID,file);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATAMAP))       return loadMap_fromFile(dataID,file);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATATREE))      return loadTree_fromFile(dataID,file);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATADOCUMENT))  return loadDocument_fromFile(dataID,file);
        
        throw new Exception("Unknown data type: "+dataType);
    }
    
    
    
    
    
    
    
    
    private Support buildDataFromPool(String dataType, String dataLocation, String dataID) throws Exception
    {
        if(!dataLocation.contains("@"))
            throw new Exception("Invalid data location: "+dataLocation);
        
        dataLocation = dataLocation.substring(5);
        File file = System2.poolFile(dataLocation);
        
        initDataAccess(dataID);
        
        if(dataType.equals(System2.RESOURCEPROVIDER_DATALIST))      return loadList_fromFile(dataID,file);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATASET))       return loadSet_fromFile(dataID,file);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATAMAP))       return loadMap_fromFile(dataID,file);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATATREE))      return loadTree_fromFile(dataID,file);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATADOCUMENT))  return loadDocument_fromFile(dataID,file);
        
        throw new Exception("Unknown data type: "+dataType);
    }
    
    
    
    
    
    
    
    
    private Support buildDataFromURL(String dataType, String dataLocation, String dataID) throws Exception
    {
        URL url = System2.url(dataLocation);
        if(url==null) throw new Exception("Unknown data location: "+dataLocation);
        
        initDataAccess(dataID);
        
        if(dataType.equals(System2.RESOURCEPROVIDER_DATALIST))      return loadList_fromURL(dataID,url);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATASET))       return loadSet_fromURL(dataID,url);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATAMAP))       return loadMap_fromURL(dataID,url);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATATREE))      return loadTree_fromURL(dataID,url);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATADOCUMENT))  return loadDocument_fromURL(dataID,url);
        
        throw new Exception("Unknown data type: "+dataType);
    }
    
    
    
    
    
    
    
    private Support buildDataFromScratch(String dataType, String dataID) throws Exception
    {
        if(dataType.equals(System2.RESOURCEPROVIDER_DATALIST))      return new SupportList(dataID);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATASET))       return new SupportSet(dataID);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATAMAP))       return new SupportMap(dataID);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATATREE))      return new SupportTree(dataID);
        if(dataType.equals(System2.RESOURCEPROVIDER_DATADOCUMENT))  return new SupportDocument(dataID);

        throw new Exception("Unknown data type: "+dataType);
    }
    
    
    
    
    
    
    
    
    private Object customizeData(String dataID, Object data)
    {
        try
        {
            Transform customizer = (Transform) System2.anonymousCall(System2.SUBSTITUTE_CUSTOMIZEDATA+"."+dataID);
            if(customizer==null) return data;
            return customizer.transform(data);
        }
        catch(Exception e){}
        return data;
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private void initDataAccess(String dataID)
    {
        try
        {
            Transform access = (Transform) System2.anonymousCall(System2.SUBSTITUTE_ACCESSFILE+"."+dataID);
            if(access==null) return;
            mapAccess.put(dataID,access);
        }
        catch(Exception e){}
    }
    
    private boolean access(String dataID)
    {
        return mapAccess.containsKey(dataID);
    }
    
    private Object loadFromAccess(String dataID, Object source) throws Exception
    {
        Transform access = (Transform) mapAccess.get(dataID);
        Provide p = (Provide)access.transform(source);
        return p.provide();
    }
    
    private void saveFromAccess(String dataID, Object source, Object data) throws Exception
    {
        Transform access = (Transform) mapAccess.get(dataID);
        Give g = (Give)access.transform(source);
        g.give(data);
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private SupportList loadList_fromFile(String dataID, File file) throws Exception
    {
        Tools_file.initFile(file);
        mapFiles.put(dataID,file);
        Object data = access(dataID)?loadFromAccess(dataID,file):Tools_file.loadList(file);
        SupportList support = new SupportList(dataID,(List)data);
        support.addActionListener(this);
        return support;
    }
    
    private SupportSet loadSet_fromFile(String dataID, File file) throws Exception
    {
        Tools_file.initFile(file);
        mapFiles.put(dataID,file);
        Object data = access(dataID)?loadFromAccess(dataID,file):Tools_file.loadSet(file);
        SupportSet support = new SupportSet(dataID,(Set)data);
        support.addActionListener(this);
        return support;
    }
    
    private SupportMap loadMap_fromFile(String dataID, File file) throws Exception
    {
        Tools_file.initFile(file);
        mapFiles.put(dataID,file);
        Object data = access(dataID)?loadFromAccess(dataID,file):Tools_file.prop(file);
        SupportMap support = new SupportMap(dataID,(Map)data);
        support.addActionListener(this);
        return support;
    }
    
    private SupportTree loadTree_fromFile(String dataID, File file) throws Exception
    {
        Tools_file.initFile(file);
        mapFiles.put(dataID,file);
        Object data = access(dataID)?loadFromAccess(dataID,file):Tools_file.readText(file);
        SupportTree support = new SupportTree(dataID,(String)data);
        support.addActionListener(this);
        return support;
    }
    
    private SupportDocument loadDocument_fromFile(String dataID, File file) throws Exception
    {
        Tools_file.initFile(file);
        mapFiles.put(dataID,file);
        Object data = access(dataID)?loadFromAccess(dataID,file):Tools_file.readText(file);
        SupportDocument support = new SupportDocument(dataID,(String)data);
        support.addActionListener(this);
        return support;
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    private SupportList loadList_fromURL(String dataID, URL url) throws Exception
    {
        Object data = access(dataID)?loadFromAccess(dataID,url):Tools_url.loadList(url);
        SupportList support = new SupportList(dataID,(List)data);
        support.addActionListener(this);
        return support;
    }
    
    private SupportSet loadSet_fromURL(String dataID, URL url) throws Exception
    {
        Object data = access(dataID)?loadFromAccess(dataID,url):Tools_url.loadSet(url);
        SupportSet support = new SupportSet(dataID,(Set)data);
        support.addActionListener(this);
        return support;
    }
    
    private SupportMap loadMap_fromURL(String dataID, URL url) throws Exception
    {
        Object data = access(dataID)?loadFromAccess(dataID,url):Tools_url.loadProperties(url);
        SupportMap support = new SupportMap(dataID,(Map)data);
        support.addActionListener(this);
        return support;
    }
    
    private SupportTree loadTree_fromURL(String dataID, URL url) throws Exception
    {
        Object data = access(dataID)?loadFromAccess(dataID,url):Tools_url.readText(url);
        SupportTree support = new SupportTree(dataID,(String)data);
        support.addActionListener(this);
        return support;
    }
    
    private SupportDocument loadDocument_fromURL(String dataID, URL url) throws Exception
    {
        Object data = access(dataID)?loadFromAccess(dataID,url):Tools_url.readText(url);
        SupportDocument support = new SupportDocument(dataID,(String)data);
        support.addActionListener(this);
        return support;
    }
    
    
    
    
    
    
    
    
    
    
    
    
    /*
     * ca sert vraiment, cette mthode ????
     */
    public void registerMap_withFile(String dataID, File file, Map map) throws Exception
    {
        Tools_file.initFile(file);
        Object data = access(dataID)?loadFromAccess(dataID,file):Tools_file.prop(file);
        map.putAll((Map)data);
        
        mapFiles.put(dataID,file);
        mapSupport.put(dataID,map);
        mapData.put(dataID,map);
    }
    
    
    
    
    
    
    
    
    
    
    public void execute() throws Exception
    {saveModifiedData();}
    
    
    
    
    
    
    
    
    
    
    private synchronized void saveModifiedData()
    {
        if(modifiedData.isEmpty()) return;
        System2.printDebug(this,"saveModifiedData",modifiedData.size()+ " data have been modified");
        Iterator it = modifiedData.iterator();
        while(it.hasNext())
        {
            String dataID = (String) it.next();
            saveData(dataID);
        }
        modifiedData.clear();
    }
    
    
    
    
    
    
    private void saveData(String dataID)
    {
        try
        {
            if(!mapFiles.containsKey(dataID)) return;
            System2.printDebug(this,"saveData(String)","save "+dataID);
            
            Support supportData = (Support) mapSupport.get(dataID);
            File file = (File) mapFiles.get(dataID);
            if(!file.canWrite())
            {System2.printDebug(this,"saveData(String)","can not write in file "+file.getAbsolutePath());return;}
            
            if(access(dataID))
            {saveFromAccess(dataID,file,supportData);return;}
            
            if(dataID.startsWith(System2.RESOURCEPROVIDER_DATALIST))
                saveList(file,(SupportList)supportData);
            
            else if(dataID.startsWith(System2.RESOURCEPROVIDER_DATASET))
                saveSet(file,(SupportSet)supportData);
            
            else if(dataID.startsWith(System2.RESOURCEPROVIDER_DATAMAP))
                saveMap(file,(SupportMap)supportData);
            
            else if(dataID.startsWith(System2.RESOURCEPROVIDER_DATATREE))
                saveTree(file,(SupportTree)supportData);
            
            else if(dataID.startsWith(System2.RESOURCEPROVIDER_DATADOCUMENT))
                saveDocument(file,(SupportDocument)supportData);
        }
        catch(Exception e)
        {System2.printError(this,"saveData(String)",e);}
    }
    
    
    
    private void saveList(File file, SupportList list) throws Exception
    {
        Tools_file.writeCollection(file,list);
    }
    
    private void saveSet(File file, SupportSet set) throws Exception
    {
        Tools_file.writeCollection(file,set);
    }
    
    private void saveMap(File file, SupportMap map) throws Exception
    {
        Properties prop = (Properties) map.getMap();
        prop.store(new FileOutputStream(file),"");
    }
    
    private void saveTree(File file, SupportTree tree) throws Exception
    {
        String value = tree.description();
        Tools_file.writeText(file,value);
    }
    
    private void saveDocument(File file, SupportDocument doc) throws Exception
    {
        String value = doc.getText(0,doc.getLength());
        Tools_file.writeText(file,value);
    }

    
    


    public synchronized void actionPerformed(ActionEvent e) 
    {
        String name = e.getSource().toString();
        modifiedData.add(name);
    }

}
